#include "Command.h"
#include "DataStructures.h"



/*
** Each of the following sets of Execute/Unexecute commands
** correspond to an "undoable" action.
*/


// --------------------------------------------------------------

bool UC_ResizeDiameter::Execute()
{
	// Get root instance
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = app->GetSession();
	Tube *tube = exp->GetTube(app->GetTube());
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_root_id );
	if(root == NULL) return false;
	RootInstance *root_inst = root->GetRootInstance(app->GetSession());
	if(root_inst == NULL) return false;
  
	// Save old radius
	RDiameter *diam = root_inst->GetDiameter(m_index);
	m_old_radius = diam->radius;

	return root_inst->ResizeDiameter(m_index, m_new_radius);
}

bool UC_ResizeDiameter::Unexecute()
{
	// Get root instance
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = app->GetSession();
	Tube *tube = exp->GetTube(app->GetTube());
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_root_id );
	if(root == NULL) return false;
	RootInstance *root_inst = root->GetRootInstance(app->GetSession());
	if(root_inst == NULL) return false;

	return root_inst->ResizeDiameter(m_index, m_old_radius);

}

// --------------------------------------------------------------

bool UC_SetRootAlive::Execute()
{
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = m_session;
  
	Tube *tube = exp->GetTube(app->GetTube());
	/// Failure if the tube is null
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_rootid );
	/// Failure if the root is null
	if(root == NULL) return false;
	RootInstance *inst = root->GetRootInstance(app->GetSession());
	/// Failure if the root instance is null
	if(inst == NULL) return false;
	/// If currently alive and to be set dead
	if(!m_alive)
	{       
		// sets all instances dead in current and future sessions
		while(session <= exp->GetMaxSessions())
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && inst->IsAlive())   inst->SetDead();
			session++;
		}
	} 
	/// If currently set dead and to be set alive
	else 
	{
		// sets all instances alive in current and past sessions
		while(session > 0)
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && !inst->IsAlive())  inst->SetAlive();
			session--;    
		}
	}  

	return true;
}

bool UC_SetRootAlive::Unexecute()
{
	ApplicationData *app = m_data->GetApplicationData();
	Experiment *exp = m_data->GetExperiment();
	int session = m_session;
  
	Tube *tube = exp->GetTube(app->GetTube());
	/// Failure if the tube is null
	if(tube == NULL) return false;
	Root *root = tube->GetRoot( m_rootid );
	/// Failure if the root is null
	if(root == NULL) return false;
	RootInstance *inst = root->GetRootInstance(app->GetSession());
	/// Failure if the root instance is null
	if(inst == NULL) return false;
	/// If currently alive and to be set dead
	if(m_alive)
	{       
		// sets all instances dead in current and future sessions
		while(session <= exp->GetMaxSessions())
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && inst->IsAlive())   inst->SetDead();
			session++;
		}
	} 
	/// If currently set dead and to be set alive
	else 
	{
		// sets all instances alive in current and past sessions
		while(session > 0)
		{
			inst = root->GetRootInstance(session);
			if (inst != NULL && !inst->IsAlive())  inst->SetAlive();
			session--;    
		}
	}  

	return true;
}

// --------------------------------------------------------------
bool UC_MoveRootSess::Execute()
{
  m_root_inst = RootSessGetRootInstance( m_root_id );
  if( m_root_inst == NULL ) return false;
  
  /// For all points
  for(int i = 0;i<m_root_inst->GetAllPoints()->size();i++)
    RootSessMovePoint(m_root_inst, i, m_root_id, m_root_inst->GetPoint(i)->x+m_offset_x,m_root_inst->GetPoint(i)->y+m_offset_y);//may need to update this to be a bit different with x and y
  
  /// Move Diameter (assumes only one diameter)
  RootSessMoveDiameter(m_root_inst, 0,m_root_id, m_root_inst->GetDiameter(0)->r_point.x + m_offset_x, m_root_inst->GetDiameter(0)->r_point.y + m_offset_y);
  
  return true;
}


bool UC_MoveRootSess::Unexecute()
{

  m_root_inst = RootSessGetRootInstance( m_root_id );
  if( m_root_inst == NULL ) return false;
  
  /// For all points
  for(int i = 0;i<m_root_inst->GetAllPoints()->size();i++)
    RootSessMovePoint(m_root_inst, i, m_root_id, m_root_inst->GetPoint(i)->x-m_offset_x,m_root_inst->GetPoint(i)->y-m_offset_y);//may need to update this to be a bit different with x and y
  
  /// Move Diameter (assumes only one diameter)
  RootSessMoveDiameter(m_root_inst, 0,m_root_id, m_root_inst->GetDiameter(0)->r_point.x - m_offset_x, m_root_inst->GetDiameter(0)->r_point.y - m_offset_y);

  return true;
}


// --------------------------------------------------------------

bool UC_MoveDiameter::Execute()
{
   
  bool success1, success2;
  
  m_root_inst = MoveDiameterGetRootInstance(m_root_id);
 
  /// Failure if root instance is null
  if(m_root_inst == NULL)     return false;
 
  // Store current diameter location for undo
  RDiameter *old_diameter = m_root_inst->GetDiameter(m_index);
  m_old_x = old_diameter->r_point.x;
  m_old_y = old_diameter->r_point.y;
  m_old_window = old_diameter->r_point.window;
 
  if(m_root_inst->MoveDiameter(m_index, m_new_x, m_new_y)) success1 = true;
  else success1 = false;

  if(m_root_inst->ChangeWindowOfDiameter(m_index, m_new_window)) success2 = true;
  else success2 = false;

  return success1 && success2;
}


bool UC_MoveDiameter::Unexecute()
{
 
  bool success1, success2;
  
  m_root_inst = MoveDiameterGetRootInstance(m_root_id);
  
  /// Failure if root instance is null
  if(m_root_inst == NULL)     return false;
  
  if(m_root_inst->MoveDiameter(m_index, m_old_x, m_old_y)) success1 = true;
  else success1 = false;

  if(m_root_inst->ChangeWindowOfDiameter(m_index, m_old_window)) success2 = true;
  else success2 = false;

  return success1 && success2;
}


// --------------------------------------------------------------

bool UC_MoveRootsInWindow::Execute()
{
  //get all roots in tube/window/session
  Experiment *exp = m_data->GetExperiment();  
  Tube *tube = exp->GetTube(m_data->GetApplicationData()->GetTube());
  
  /// Failure if tube is null
  if( tube == NULL ) return false;
  
  /// Get all roots
  vector<Root> *roots = tube->GetRootsInWindow(m_window);
  if (roots)
  {
    vector<Root>::iterator iter;
    //for each root
    for(iter = roots->begin();iter != roots->end();iter++)
      RootWindowMoveRootSess(iter->GetID(),m_data->GetApplicationData()->GetSession(), m_offset_x, m_offset_y);
  }

  return true;
}


bool UC_MoveRootsInWindow::Unexecute()
{
  //get all roots in tube/window/session
  Experiment *exp = m_data->GetExperiment();  
  Tube *tube = exp->GetTube(m_data->GetApplicationData()->GetTube());
  
  /// Failure if tube is null
  if( tube == NULL ) return false;
  
  /// Get all roots
  vector<Root> *roots = tube->GetRootsInWindow(m_window);
  if (roots)
  {
    vector<Root>::iterator iter;
    //for each root
    for(iter = roots->begin();iter != roots->end();iter++)
      RootWindowMoveRootSess(iter->GetID(),m_data->GetApplicationData()->GetSession(), -m_offset_x, -m_offset_y);
  }

  return true;
}


// --------------------------------------------------------------

bool UC_SetRootColor::Execute() 
{
  ApplicationData *app = m_data->GetApplicationData();
  Experiment *exp = m_data->GetExperiment();
  
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if the tube is null
  if(tube == NULL) return false;
  Root *root = tube->GetRoot( m_root_id );
  /// Failure if the root is null
  if(root == NULL) return false;
  RootInstance *inst = root->GetRootInstance(app->GetSession());
  /// Failure if the root instance is null
  if(inst == NULL) return false;

  m_old_color = inst->GetColor();
  if(m_old_color != m_color_choice)
  {
    /// Set status unsaved
    inst->SetColor(m_color_choice);
	return true;

  }
  else return false;

}

bool UC_SetRootColor::Unexecute() 
{
  ApplicationData *app = m_data->GetApplicationData();
  Experiment *exp = m_data->GetExperiment();
  
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if the tube is null
  if(tube == NULL) return false;
  Root *root = tube->GetRoot( m_root_id );
  /// Failure if the root is null
  if(root == NULL) return false;
  RootInstance *inst = root->GetRootInstance(app->GetSession());
  /// Failure if the root instance is null
  if(inst == NULL) return false;


  inst->SetColor(m_old_color);
  return true;
}