#ifndef __IMAGE_FILE_FORMAT_H__
#define __IMAGE_FILE_FORMAT_H__

#include <string>
#include <vector>
#include <assert.h>


using namespace std;


/** These are the NUM_OF_ELEMENTS elements of which any possible image filename is composed */
typedef enum 
{
  EXPT,             ///< Experiment name
  TUBE,             ///< Tube identifier, usually a number
  WINDOW,           ///< Window number
  SESSION,          ///< Session number
  SDATE,            ///< Date
  TIME,             ///< Time
  INITIALS,         ///< Initials of data gatherer
  UNUSED1,          ///< Skips unnecessary element in filename
  UNUSED2,          ///< Skips unnecessary element in filename
  UNUSED3,          ///< Skips unnecessary element in filename
  UNUSED4,          ///< Skips unnecessary element in filename
  NUM_OF_ELEMENTS   ///< Total number of elements that may be found
} ELEMENT;

// used to help with ordering the date elements
typedef enum
{
  YEAR,
  MONTH,
  DAY
} DATE_ORDER;

/** contains information about a specific element in the image filename */
struct OrderElement
{
  ELEMENT element;  ///< integer indicating which element
  CString prefix;   ///< prefix (if it exists) in the actual image filename; the prefix is thrown away upon parsing
  int length;       ///< maximum number of characters in the image filename devoted to this element -- used with tube, window, session
  bool included;    ///< true if element is included in image filename

  /** @returns a human-readable form of ELEMENT */
  CString GetName() const
  {
    CString name;
    switch(element)
    {
      case EXPT : name = "Experiment"; break;
      case TUBE : name = "Tube"; break;
      case WINDOW : name = "Window"; break;
      case SESSION : name = "Session"; break;
      case SDATE : name = "Date"; break;
      case TIME : name = "Time"; break;
      case INITIALS : name = "Initials"; break;
      case UNUSED1 : name = "Unused 1"; break;
      case UNUSED2 : name = "Unused 2"; break;
      case UNUSED3 : name = "Unused 3"; break;
      case UNUSED4 : name = "Unused 4"; break;
      default:  assert(0);  name = "[error]"; break;
    }
    return name;
  }
};

/** contains the format of the date */
struct Date_Format
{  
  std::vector<int> order; // used to determine the order
  bool four_year;
  CString date_delim;
};

/** contains information about the image file format used in the current directory
    E.g., order of the items, delimeter between items, etc.
*/ 
struct ImageFileFormatInfo
{
  vector<OrderElement> InfoOrder;  ///< order of the elements in the filename
                                   ///  vector always includes NUM_OF_ELEMENTS
  int numElementsInFileName;       ///< number of elements included in the filename
  CString filetype;                ///< image type, e.g., jpg or bmp
  char delim;                      ///< delimiter between elements in the filename
  Date_Format m_date_format;
};

/** contains the date */
struct Date
{
  int year;   ///< year
  int month;  ///< month
  int day;    ///< day

  // Unused at the moment
  char date_delim;
};

/** contains information found in the image file name */
struct ImageInfo
{
  CString expt;      ///< Experiment name
  CString tube;      ///< Tube
  int window;        ///< Window number
  int session;       ///< Session number
  Date date;         ///< Date that the image was taken - contains day, month, year
  CString initials;  ///< Initials of data gatherer (Bartz format)
};

/** Stores image file format, and provides method to retrieve file name. 
*/
class ImageFileFormat
{
public:
  ImageFileFormat();
  virtual ~ImageFileFormat() {}

  /** Generates a filename given the parameters, then searches for the file in
      the given directory.
      @param directory Directory containing the images
      @param tube Tube # of the image
      @param window Window # of the image
      @param session Session # of the image
      @returns filename of the corresponding file or an empty string if the file
               is not found
      @bug:  This function really should be const.  See the comments in .cpp file. -- STB
  */
  CString GetFileName(const CString& directory, const CString& tube, int window, int session);

  /** Searches a directory for a file that matches the given image filename format
      @param directory The directory to search
      @returns true if file is found; false otherwise
  */
  bool ImageFileExistsInDirectory(const CString& directory) const;

  /* Determines the maximum and minimum window and session
     @param directory The directory of images
     @param max_window Pointer to maximum window value
     @param min_window Pointer to minimum window value
     @param max_session Pointer to maximum session value
     @param min_session Pointer to minimum session value
     @returns false if error, true otherwise
  */
  bool FindNavigationLimits(const CString& directory, int* max_window, int* min_window, int* max_session,int* min_session) const;

  // same as above except with an added prototype progress bar
  bool FindNavigationLimits(const CString& directory, int* max_window, int* min_window, int* max_session,int* min_session, CWnd *pParentWnd) const;

  int CountFilesInDirectory(const CString& directory) const;

  /* Gets image info from a filename
     @param filename The filename from which the image info should be obtained
     @returns a struct containing the image information
  */
  ImageInfo GetImageInfo(const CString& filename) const;

  /* Gets image info of the first valid file found in a directory
     @param directory The image directory
     @returns a struct containing the image information of the first file
  */
  ImageInfo GetImageInfoOfFirstFile(const CString& directory) const;

  /* Sets the image file format info given a const reference */
  void SetImageFileFormatInfo(const ImageFileFormatInfo&);

  /* @returns a const reference to an imagefileformatinfo struct */
  const ImageFileFormatInfo& GetImageFileFormatInfo() const;

  /* @returns true if date information is available in file name */
  bool DateIsIncluded() const;  

  /* Sets the format to the Bartz format */
  void SetToBartz();

  /* Sets the forat for the date to an default state */
  void InitializeDateFormat();

  /* Writes string containing image file format to a given file */
  void Save(FILE* fp) const;

  /* Loads image file format from a given file */
  char* Load(FILE* fp);
  
  /* Gets the first image in the directory */
  CString GetFirstImage(const CString& directory) const;

  /* Gets the rectangle that is the same size as the image */
  static CRect GetImageRectangle(const CString& filename);

  // gets dates from file. date must be included in filename
  vector<Date> GetDatesFromFile(const CString& filename, int max_session);  

  // gets dates from file. date must be included in filename also includes a progress bar
  vector<Date> GetDatesFromFile(const CString& dir, int max_sessions, CWnd *pParentWnd);

private:
  ImageFileFormatInfo m_image_file_format_info;
  vector<Date> m_dates_from_file;            // keeps track of dates if it is included in the file

  /* @returns a search string created from the image file format */
  CString GetFileSearchString() const;

  /* Sets the lengths for the elements included in the file name
     @returns false if error, true otherwise
  */
  bool SetLengths(const CString& file_directory);
};

#endif __IMAGE_FILE_FORMAT_H__